// core_base.shader

#if !defined(RUNTIME)
#   define VERTEX_SHADER
#endif

#if defined(VERTEX_SHADER)

uniform mat4 worldMat;
//uniform mat3 normalMat;
uniform mat4 viewMat;
//uniform mat4 cameraMat;
uniform mat4 projMat;

uniform lowp vec4 meshColor; 

vec4 getPosWSLS(vec4 posLS)
{
    return worldMat * posLS;
}

vec4 getPosVSLS(vec4 posLS)
{
    return viewMat * (worldMat * posLS);
}

vec4 getPosVSWS(vec4 posWS)
{
    return viewMat * posWS;
}

vec4 getPosSSLS(vec4 posLS)
{
    return projMat * (viewMat * (worldMat * posLS));
}

vec4 getPosSSWS(vec4 posWS)
{
    return projMat * (viewMat * posWS);
}

vec4 getPosSSVS(vec4 posVS)
{
    return projMat * posVS;
}

vec3 getNrmWSLS(vec3 nrmLS)
{
    return normalize( mat3(worldMat[0].xyz, worldMat[1].xyz, worldMat[2].xyz) * nrmLS);
//    return normalMat * vertNormal;
}

vec4 getVertexColor(vec3 vertexColor)
{
    return vec4(vertexColor.rgb/255.0, 1.0);
}

vec4 getVertexColor(vec4 vertexColor)
{
    return vertexColor/255.0;
}

mediump vec2 getEnvironmentTexCoord(vec3 nrmWS)
{
    vec3 nrmVS = normalize( mat3(viewMat[0].xyz, viewMat[1].xyz, viewMat[2].xyz) * nrmWS);
    nrmVS.y = -nrmVS.y;
    nrmVS.z = max(0.0, nrmVS.z);

    return normalize(nrmVS).xy * 0.5 + vec2(0.5);
}

#endif

lowp vec4 getSaturateColor(lowp vec4 color, lowp float saturateColor)
{
    lowp float greyScale = dot(color.rgb, vec3(0.2989, 0.5870, 0.1140));
    
    return vec4(mix(vec3(greyScale), color.rgb, saturateColor), color.a);
}
