// core_light.shader

#if defined(VERTEX_SHADER)

#define DIRECTIONAL_LIGHT
//#define POINT_LIGHT


#if defined(POINT_LIGHT) 

uniform highp vec4 light0Pos;
uniform highp vec4 light1Pos;
uniform highp vec4 light2Pos;
uniform highp vec4 light3Pos;

lowp vec4 getLightPoint(vec3 posWS, vec3 nrmWS, highp vec4 light0Position, highp vec4 light1Position, highp vec4 light2Position, highp vec4 light3Position)
{
    vec3 light0Dir = light0Position.xyz - posWS.xyz;
    vec3 light1Dir = light1Position.xyz - posWS.xyz;
    vec3 light2Dir = light2Position.xyz - posWS.xyz;
    vec3 light3Dir = light3Position.xyz - posWS.xyz;

    lowp vec4 len = vec4( length(light0Dir.xyz),
        length(light1Dir.xyz),
        length(light2Dir.xyz),
        length(light3Dir.xyz) );

    lowp vec4 v = max( vec4(dot(nrmWS, light0Dir.xyz),
        dot(nrmWS, light1Dir.xyz),
        dot(nrmWS, light2Dir.xyz),
        dot(nrmWS, light3Dir.xyz)) / len, vec4(0.0) );

    return (vec4(1.0) - min(vec4(len * vec4(light0Position.w, light1Position.w, light2Position.w, light3Position.w)), vec4(1.0))) * v;
}

#elif defined(DIRECTIONAL_LIGHT) 

uniform highp vec3 light0Direction;
uniform highp vec3 light1Direction;
uniform highp vec3 light2Direction;
uniform highp vec3 light3Direction;

lowp vec4 getLightDirectional(vec3 nrmWS, highp vec3 light0Dir, highp vec3 light1Dir, highp vec3 light2Dir, highp vec3 light3Dir)
{
    return max( vec4( dot(nrmWS, light0Dir),
        dot(nrmWS, light1Dir),
        dot(nrmWS, light2Dir),
        dot(nrmWS, light3Dir)), vec4(0.0) );
}

#endif         


uniform mediump vec3 light0Color;
uniform mediump vec3 light1Color;
uniform mediump vec3 light2Color;
uniform mediump vec3 light3Color;

uniform mediump vec3 materialEmit; // Emit, EmitStrength, 0.0

lowp vec3 getLight(vec3 posWS, vec3 nrmWS)
{
    lowp vec4 intensity;
    
#if defined(POINT_LIGHT) 
    intensity = getLightPoint(posWS, nrmWS, light0Pos, light1Pos, light2Pos, light3Pos);
#elif defined(DIRECTIONAL_LIGHT) 
    intensity = getLightDirectional(nrmWS, light0Direction, light1Direction, light2Direction, light3Direction);
#else
    intensity = vec4(1.0);
#endif         

    vec3 lights = light0Color*intensity.x + light1Color*intensity.y + light2Color*intensity.z + light3Color*intensity.w;

    return mix(lights, vec3(materialEmit.y), materialEmit.x);
}

#endif
