// model_refmaskemit.shader

#include "shaders/core_base.shader"
#include "shaders/core_fog.shader"
#include "shaders/core_light.shader"


varying mediump vec2 diffuseTexCoord;
varying mediump vec2 fogTexCoord;
varying mediump vec2 environmentTexCoord;

varying lowp vec4 lightColor;


#if defined(VERTEX_SHADER)

attribute vec4 vertPos;
attribute vec3 vertColor;
attribute vec3 vertNormal;
attribute vec2 texCoords0;

void main()
{
    vec4 posWS = getPosWSLS(vertPos);
    vec4 posVS = getPosVSWS(posWS);
    gl_Position = getPosSSVS(posVS);

    vec3 nrmWS = getNrmWSLS(vertNormal);

    lowp vec4 vertexColor = getVertexColor(vertColor);
    lightColor = vec4(getLight(posWS.xyz, nrmWS) * vertexColor.rgb, vertexColor.a) * meshColor;

    diffuseTexCoord = texCoords0;
    fogTexCoord = getFogTexCoord(posVS);
    environmentTexCoord = getEnvironmentTexCoord(nrmWS);
}

#endif


#if defined(FRAGMENT_SHADER)

uniform sampler2D tex0; // Diffuse
uniform sampler2D tex1; // Fog
uniform sampler2D tex2; // Environment
uniform sampler2D tex3; // MaskEmit

void main()
{
    lowp vec4 diffuseColor = texture2D(tex0, diffuseTexCoord);
    lowp vec4 fogColor = texture2D(tex1, fogTexCoord);
    lowp vec4 environmentColor = texture2D(tex2, environmentTexCoord);
    lowp vec4 maskemitColor = texture2D(tex3, diffuseTexCoord);

    lowp vec4 color = diffuseColor;
    color *= lightColor;
    color.rgb = mix(color.rgb, environmentColor.rgb, environmentColor.a*maskemitColor.a);
    color.rgb = mix(color.rgb, fogColor.rgb, fogColor.a);
    color.rgb += maskemitColor.rgb;
    
    gl_FragColor = color;
}

#endif
