// spritexyz.shader

#include "shaders/core_base.shader"
#include "shaders/core_fog.shader"
#include "shaders/core_light.shader"


varying mediump vec2 diffuseTexCoord;
varying mediump vec2 fogTexCoord;

varying lowp vec4 lightColor;


#if defined(VERTEX_SHADER)

attribute vec4 vertPos;
attribute vec3 vertColor;
attribute vec3 vertNormal;
attribute vec2 texCoords0;

void main()
{
//    vec4 posWS = vec4(cameraMat[0].xyz * vertPos.x + cameraMat[2].xyz * vertPos.y + cameraMat[1].xyz * vertPos.z, 1.0);
//    posWS.xyz += vec3(worldMat[3].x, worldMat[3].y, worldMat[3].z);

    vec4 posWS = vec4( vec3(viewMat[0].x, viewMat[1].x, viewMat[2].x) * vertPos.x
        + vec3(viewMat[0].z, viewMat[1].z, viewMat[2].z) * vertPos.y
        + vec3(viewMat[0].y, viewMat[1].y, viewMat[2].y) * vertPos.z,
        1.0); // mat3 cameraMat = transpose(mat3(viewMat))
        
    posWS.xyz += vec3(worldMat[3].x, worldMat[3].y, worldMat[3].z);
    
    vec4 posVS = getPosVSWS(posWS);
    gl_Position = getPosSSVS(posVS);

    vec3 nrmWS = getNrmWSLS(vertNormal);
    
    float alpha = max(0.0, normalize( mat3(viewMat[0].xyz, viewMat[1].xyz, viewMat[2].xyz) * nrmWS).z);
//    alpha = alpha*alpha;
    
    lowp vec4 vertexColor = getVertexColor(vertColor);
    lightColor = vec4(getLight(posWS.xyz, nrmWS) * vertexColor.rgb, vertexColor.a*alpha) * meshColor;

    diffuseTexCoord = texCoords0;
    fogTexCoord = getFogTexCoord(posVS);
}

#endif


#if defined(FRAGMENT_SHADER)

uniform sampler2D tex0; // Diffuse
uniform sampler2D tex1; // Fog

void main()
{
    lowp vec4 diffuseColor = texture2D(tex0, diffuseTexCoord);
    lowp vec4 fogColor = texture2D(tex1, fogTexCoord);

    lowp vec4 color = diffuseColor*lightColor;

    gl_FragColor = vec4(mix(color.rgb, fogColor.rgb, fogColor.a) * color.a, color.a);
}

#endif
